<?php

//
// Page Setup
//

define('IN_GWEFAN', true);

require_once(dirname(__FILE__).'/config.php');

$gwefan = new GwefanAPI(GWEFAN_DATABASE_PATH);
$gwefan->setReadOnly(false); //Put Gwefan in "admin mode".

//
// Authentification
//

//Catch /ACTION/logout
if( isset($_GET['top']) && isset($_GET['sub']) && $_GET['top'] == 'ACTION' && $_GET['sub'] == 'logout' ) {
    header('HTTP/1.1 401 Unauthorized');
    $redirect_link = $gwefan->makelink('');
    $p401 = file_get_contents(GWEFAN_RESOURCES_PATH.'/error_pages/401_logout.html');
    print str_replace('{$redirect_link}', $redirect_link, $p401);
    exit(1);
}

//Authenticate
$auth = false;
switch(GWEFAN_LOGIN_METHOD)
{
    case GWEFAN_AUTH_BASIC:
	$auth = $gwefan->requestBasicAuth();
	break;
    case GWEFAN_AUTH_DIGEST:
	$auth = $gwefan->requestDigestAuth();
	break;
    case GWEFAN_AUTH_PHP:
	throw new Exception('GWEFAN_AUTH_PHP is not implemented yet, sorry.');
	break;
    default:
	throw new Exception('Undefined value for constant GWEFAN_LOGIN_METHOD.');
}
if( !$auth )
{
    @header('HTTP/1.1 401 Unauthorized');
    //User cancelled authentification or gave wrong credentials
    readfile(GWEFAN_RESOURCES_PATH.'/error_pages/401.html');
    exit(1);
}

//Start a session. This is necessary for KCFinder integration, so it knows that
//the user is logged in...
session_start();
$_SESSION['KCFINDER'] = array();
$_SESSION['KCFINDER']['disabled'] = false;

//
// Handle requests
//

if( isset($_POST['action']) )
{
    switch($_POST['action'])
    {
	case 'new_toplink':
	    $gwefan->createToplink();
	    break;
	case 'new_sublink':
	    $gwefan->createSublink($_POST['toplink']);
	    break;
	case 'move_page_up':
	    if( isset($_POST['toplink']) )
	    {
		if( isset($_POST['sublink']) )
		{
		    $gwefan->pageMoveUp($_POST['toplink'], $_POST['sublink']);
		} else {
		    $gwefan->pageMoveUp($_POST['toplink']);
		}
		//Refresh page to show changed order..
		$current_path = $gwefan->getCurrentPath();
		$top_path = array_pop($current_path);
		header('Location: '.$top_path['link']);
		exit;
	    }
	    break;
	case 'move_page_down':
	    if( isset($_POST['toplink']) )
	    {
		if( isset($_POST['sublink']) && $gwefan->pageCanMoveDown($_POST['toplink'], $_POST['sublink']) )
		{
		    $gwefan->pageMoveDown($_POST['toplink'], $_POST['sublink']);
		} else {
		    $gwefan->pageMoveDown($_POST['toplink']);
		}
		//Refresh page to show changed order..
		$current_path = $gwefan->getCurrentPath();
		$top_path = array_pop($current_path);
		header('Location: '.$top_path['link']);
		exit;
	    }
	    break;
	case 'delete_page':
	    if( !isset($_POST['sublink']) )
	    {
		$gwefan->deleteToplink($_POST['toplink']);
		//Redirect to HOME
		$current_path = $gwefan->getCurrentPath();
		header('Location: '.$current_path[0]['link']);
		exit;
	    } else {
		$gwefan->deleteSublink($_POST['toplink'], $_POST['sublink']);
		//Redirect to toplink
		$current_path = $gwefan->getCurrentPath();
		header('Location: '.$current_path[1]['link']);
		exit;
	    }
	    break;
	case 'page_change_identifier':
	    $smarty->assign('confirm_change_identifier', true);
	    break;
	case 'update_page':
	    //print "<pre>";
		//var_export($_POST);
	    //print "</pre>";
	    $contents = array(
		'content'	=>	$_POST['page_content'],
		'name'		=>	$_POST['page_name'],
		'identifier'	=>	$_POST['page_identifier'],
		//'is_include'	=>	$_POST['page_is_include'],
	    );
	    if( !isset($_POST['sublink']) )
	    {
		$gwefan->updatePage($contents, $_POST['toplink']);
	    } else {
		$gwefan->updatePage($contents, $_POST['toplink'], $_POST['sublink']);
	    }
	    break;
    }
}

//
// General Data
//

$setup = $gwefan->getSetup();

$smarty->assign('sitename', $setup['sitename']);
$smarty->assign('resources_path', GWEFAN_REL_RESOURCES_PATH);
$smarty->assign('viewer_path', GWEFAN_REL_PATH);

//Find toplinks
$toplinks = $gwefan->getToplinks();
$smarty->assign('toplinks', $toplinks);

//Find sublinks
$smarty->assign('has_sublinks', $gwefan->pageHasSublinks());
$sublinks = $gwefan->getSublinks();
$smarty->assign('sublinks', $sublinks);

//Find current path
$smarty->assign('paths', $gwefan->getCurrentPath());

//
// Page Content
//

$page_content = $gwefan->getCurrentPage();

//Can this page be deleted?
$page_content['can_delete'] = $gwefan->pageCanDelete(
    $page_content['toplink']['identifier'],
    empty($page_content['sublink']['identifier']) ? null : $page_content['sublink']['identifier']
);

//Can this page move up or down?
$page_content['can_move_up'] = $gwefan->pageCanMoveUp(
    $page_content['toplink']['identifier'],
    empty($page_content['sublink']['identifier']) ? null : $page_content['sublink']['identifier']
);
$page_content['can_move_down'] = $gwefan->pageCanMoveDown(
    $page_content['toplink']['identifier'],
    empty($page_content['sublink']['identifier']) ? null : $page_content['sublink']['identifier']
);

$smarty->assign('current_page', $page_content);

//
// Display Template
//

$smarty->display('admin.tpl');
?>
