<?php
class GwefanAPI
{
    //
    //Constants
    //
    
    const VIEWER_PREFIX = '/view';
    const EDITOR_PREFIX = '/edit';
    
    //
    //Public variables
    //
    
    public $db = null;
    
    //
    //Protected variables
    //
    
    protected $readonly = true;
    protected $rel_path = '/';
    protected $cfg_rewrite = false;
    protected $toplinks = null;
    protected $current_path = null;
    protected $sublinks = null;
    protected $current_page = null;
    protected $setup = null;
    protected $link_prefix = self::VIEWER_PREFIX;
    
    //
    //Methods
    //
    
    public function __construct($db_path)
    {
	//Connect to database
	$this->db = new PDO('sqlite:'.$db_path);
	
	//Paths and rewriting
	if( defined('GWEFAN_REL_PATH') )
	{
	    $this->rel_path = GWEFAN_REL_PATH;
	}
	if( defined('GWEFAN_CFG_REWRITE') )
	{
	    $this->cfg_rewrite = (bool) GWEFAN_CFG_REWRITE;
	}
    }
    
    public function requestBasicAuth()
    {
	//$users must be array of type user => md5(password)
	//e.g. $users = array('admin' => 'mypass', 'guest' => 'guest');
	$setup = $this->getSetup();
	$users = array($setup['username'] => $setup['password']);
	
	$realm = 'Restricted Access';
	
	if( empty($_SERVER['PHP_AUTH_USER']) )
	{
	    header('HTTP/1.1 401 Unauthorized');
	    header('WWW-Authenticate: Basic realm='.$realm);
	    
	    return false;
	} else {
	    //Check credentials..
	    if( isset($users[$_SERVER['PHP_AUTH_USER']])
		&& $users[$_SERVER['PHP_AUTH_USER']] == $_SERVER['PHP_AUTH_PW'] )
	    {
		return true;
	    }
	    return false;
	}
    }
    
    public function requestDigestAuth()
    {
	//$users must be array of type user => md5(password)
	//e.g. $users = array('admin' => 'mypass', 'guest' => 'guest');
	$setup = $this->getSetup();
	$users = array($setup['username'] => $setup['password']);
	
	$realm = 'Restricted Access';

	if( empty($_SERVER['PHP_AUTH_DIGEST']) )
	{
	    header('HTTP/1.1 401 Unauthorized');
	    header('WWW-Authenticate: Digest realm="'.$realm.'",qop="auth",nonce="'.uniqid().'",opaque="'.md5($realm).'"');

	    return false;
	} else {
	    // analyze the PHP_AUTH_DIGEST variable
	    $data = self::parseDigestAuth($_SERVER['PHP_AUTH_DIGEST']);
	    if( !$data || !isset($users[$data['username']]) )
	    {
		return false;
	    }

	    if( $data['response'] != self::createDigestResponse($data, $realm, $users) )
	    {
		return false;
	    }

	    // ok, valid username & password
	    return $data;
	}
    }
    
    public static function createDigestResponse($data, $realm, $users)
    {
	// generate the valid response
	$A1 = md5($data['username'].':'.$realm.':'.$users[$data['username']]);
	$A2 = md5($_SERVER['REQUEST_METHOD'].':'.$data['uri']);
	return md5($A1.':'.$data['nonce'].':'.$data['nc'].':'.$data['cnonce'].':'.$data['qop'].':'.$A2);
    }
    
    public static function parseDigestAuth($text)
    {
        // protect against missing data
	$needed_parts = array('nonce'=>1, 'nc'=>1, 'cnonce'=>1, 'qop'=>1, 'username'=>1, 'uri'=>1, 'response'=>1);
	$data = array();
	$keys = implode('|', array_keys($needed_parts));

	preg_match_all('@(' . $keys . ')=(?:([\'"])([^\2]+?)\2|([^\s,]+))@', $text, $matches, PREG_SET_ORDER);

	foreach ($matches as $m)
	{
	    $data[$m[1]] = $m[3] ? $m[3] : $m[4];
	    unset($needed_parts[$m[1]]);
	}

	return $needed_parts ? false : $data;
    }
    
    public function setReadOnly($readonly)
    {
	$this->readonly = (bool) $readonly;
	if( $this->readonly )
	{
	    $this->link_prefix = self::VIEWER_PREFIX;
	} else {
	    $this->link_prefix = self::EDITOR_PREFIX;
	}
    }
    
    public function getReadOnly()
    {
	return $this->readonly;
    }
    
    public function getDb()
    {
	return $this->db;
    }
    
    public function setRelPath($rel_path)
    {
	$this->rel_path = $rel_path;
    }
    
    public function getRelPath()
    {
	return $this->rel_path;
    }
    
    public function setCfgRewrite($cfg_rewrite)
    {
	$this->cfg_rewrite = (bool) $cfg_rewrite;
    }
    
    public function getCfgRewrite()
    {
	return $this->cfg_rewrite;
    }
    
    public function makelink($toplink, $sublink=null)
    {
	$link = GWEFAN_REL_PATH;
	if(GWEFAN_CFG_REWRITE == true)
	{
	    $link .= $this->link_prefix;
	    $link .= '/'.$toplink;
	    if($sublink !== null)
	    {
		$link .= '/'.$sublink;
	    }
	} else {
	    $link .= basename($_SERVER['SCRIPT_NAME']);
	    $link .= '?top='.$toplink;
	    if($sublink !== null)
	    {
		$link .= '&sub='.$sublink;
	    }
	}
	return $link;
    }
    
    protected function fetchSetup()
    {
	$results = $this->db->query('SELECT * FROM `setup`');
	$results = $results->fetchAll(PDO::FETCH_ASSOC);
	$this->setup = array();
	foreach($results as $result)
	{
	    $this->setup[$result['key']] = $result['value'];
	}
    }
    
    public function getSetup()
    {
	if( $this->setup === null )
	{
	    $this->fetchSetup();
	}
	return $this->setup;
    }
    
    protected function fetchToplinks()
    {
	$results = $this->db->query('SELECT * FROM `toplinks` ORDER BY `order`');
	$results = $results->fetchAll(PDO::FETCH_ASSOC);
	$this->toplinks = array();
	foreach($results as $result)
	{
	    $result['link'] = $this->makelink($result['identifier']);
	    $this->toplinks[] = $result;
	}
    }
    
    public function getToplinks()
    {
	if( $this->toplinks === null )
	{
	    $this->fetchToplinks();
	}
	return $this->toplinks;
    }
    
    protected function fetchSublinks()
    {
	$toplink = $this->getCurrentPage();
	$toplink = $toplink['toplink'];
	$results = $this->db->prepare('SELECT * FROM `sublinks` WHERE `parent` = :parent ORDER BY `order`');
	$results->bindValue(':parent', $toplink['id']);
	$results->execute();
	$results = $results->fetchAll(PDO::FETCH_ASSOC);
	$this->sublinks = array();
	foreach($results as $result)
	{
	    $result['link'] = $this->makelink($toplink['identifier'], $result['identifier']);
	    $this->sublinks[] = $result;
	}
    }
    
    public function getSublinks()
    {
	if( $this->sublinks === null )
	{
	    $this->fetchSublinks();
	}
	return $this->sublinks;
    }
    
    public function pageHasSublinks()
    {
	$sublinks = $this->getSublinks();
	if( empty($sublinks) )
	{
	    return false;
	} else {
	    return true;
	}
    }
    
    protected function fetchCurrentPath()
    {
	$home = $this->getToplinks();
	$home = $home[0];
	$this->current_path = array($home); //Adds "Home" to paths
	if( isset($_GET['top']) )
	{
	    $result = $this->db->prepare('SELECT * FROM `toplinks` WHERE `identifier` = :identifier');
	    $result->bindValue(':identifier', $_GET['top'], PDO::PARAM_STR);
	    $result->execute();
	    $result = $result->fetch(PDO::FETCH_ASSOC);
	    if( $result !== false && $result['id'] != 1 )  //exclude nonexisting path and "Home"
	    {
		$result['link'] = $this->makelink($result['identifier']);
		$this->current_path[] = $result;
		$toplink = $result;
		if( isset($_GET['sub']) )
		{
		    $result = $this->db->prepare('SELECT * FROM `sublinks` WHERE `identifier` = :identifier AND `parent` = :parent');
		    $result->bindValue(':identifier', $_GET['sub'], PDO::PARAM_STR);
		    $result->bindValue(':parent', $toplink['id'], PDO::PARAM_INT);
		    $result->execute();
		    $result = $result->fetch(PDO::FETCH_ASSOC);
		    if( $result !== false )  //exclude nonexisting path
		    {
			$result['link'] = $this->makelink($toplink['identifier'], $result['identifier']);
			$this->current_path[] = $result;
		    }
		}
	    }
	}
    }
    
    public function getCurrentPath()
    {
	if( $this->current_path === null )
	{
	    $this->fetchCurrentPath();
	}
	return $this->current_path;
    }
    
    protected function fetchCurrentPage()
    {
	$current_path = $this->getCurrentPath();
	$this->current_page = array();
	if( count($current_path) >= 3 ) //This must be a sublink
	{
	    $this->current_page['sublink'] = array_pop($current_path);
	    $this->current_page['title'] = $this->current_page['sublink']['name'];
	    $result = $this->db->prepare('SELECT * FROM `pages` WHERE `sublink` = :sublink');
	    $result->bindValue(':sublink', $this->current_page['sublink']['id'], PDO::PARAM_INT);
	    $result->execute();
	    $result = $result->fetch(PDO::FETCH_ASSOC);
	    $this->current_page = array_merge($result, $this->current_page);
	}
	$this->current_page['toplink'] = array_pop($current_path);
	if( !isset($this->current_page['title']) )
	{
	    $this->current_page['title'] = $this->current_page['toplink']['name'];
	    $result = $this->db->prepare('SELECT * FROM `pages` WHERE `toplink` = :toplink');
	    $result->bindValue(':toplink', $this->current_page['toplink']['id'], PDO::PARAM_INT);
	    $result->execute();
	    $result = $result->fetch(PDO::FETCH_ASSOC);
	    $this->current_page = array_merge($result, $this->current_page);
	}
    }
    
    public function getCurrentPage()
    {
	if( $this->current_page === null )
	{
	    $this->fetchCurrentPage();
	}
	return $this->current_page;
    }
    
    protected function fetchLinks($toplink, $sublink=null)
    {
	if( !$this->pageExists($toplink, $sublink) )
	{
	    return false;
	}
	$links = array('toplink' => array(), 'sublink' => array());
	//Fetch toplink
	$result = $this->db->prepare('SELECT * FROM `toplinks` WHERE `identifier` = :identifier');
	$result->bindValue(':identifier', $toplink, PDO::PARAM_STR);
	$result->execute();
	$links['toplink'] = $result->fetch(PDO::FETCH_ASSOC);
	if( $sublink != null )
	{
	    $result = $this->db->prepare('SELECT * FROM `sublinks` WHERE `identifier` = :identifier AND `parent` = :parent');
	    $result->bindValue(':parent', $links['toplink']['id'], PDO::PARAM_INT);
	    $result->bindValue(':identifier', $sublink, PDO::PARAM_STR);
	    $result->execute();
	    $links['sublink'] = $result->fetch(PDO::FETCH_ASSOC);
	}
	return $links;
    }
    
    public function pageExists($toplink, $sublink=null)
    {
	//Check toplink
	$result = $this->db->prepare('SELECT `id` FROM `toplinks` WHERE `identifier` = :identifier');
	$result->bindValue(':identifier', $toplink, PDO::PARAM_STR);
	$result->execute();
	$result = $result->fetch(PDO::FETCH_ASSOC);
	if( !$result )
	{
	    return false; //Toplink not found
	} else {
	    $toplink = $result['id'];
	}
	if( $sublink != null )
	{
	    $result = $this->db->prepare('SELECT `id` FROM `sublinks` WHERE `identifier` = :identifier AND `parent` = :parent');
	    $result->bindValue(':parent', $toplink, PDO::PARAM_INT);
	    $result->bindValue(':identifier', $sublink, PDO::PARAM_STR);
	    $result->execute();
	    $result = $result->fetch(PDO::FETCH_ASSOC);
	    if( !$result )
	    {
		return false; //Sublink not found
	    }
	}
	return true;
    }
    
    public function pageCanMoveUp($toplink, $sublink=null)
    {
	//Get page details...
	$page = $this->fetchLinks($toplink, $sublink);
	if( !$page )
	{
	    return false; //Page doesn't exist..
	}
	//Is this "Home"?
	if( $page['toplink']['id'] == 1 )
	{
	    return false;
	}
	//Is this a toplink or a sublink?
	if( empty($page['sublink']) )
	{
	    //Okay, toplink! Is it the first after Home?
	    if( count($this->getToplinks()) > 1 && $this->toplinks[1]['id'] == $page['toplink']['id'] )
	    {
		return false;
	    }
	} else {
	    //Okay, sublink! Is it the first of them?
	    if( count($this->getSublinks()) > 1 && $this->sublinks[0]['id'] == $page['sublink']['id'] )
	    {
		return false;
	    }
	}
	//No impediments found...
	return true;
    }
    
    public function pageCanMoveDown($toplink, $sublink=null)
    {
	//Get page details...
	$page = $this->fetchLinks($toplink, $sublink);
	if( !$page )
	{
	    return false; //Page doesn't exist..
	}
	//Is this "Home"?
	if( $page['toplink']['id'] == 1 )
	{
	    return false;
	}
	//Is this a toplink or a sublink?
	if( empty($page['sublink']) )
	{
	    //Ok, toplink. Is it the last one?
	    if( $this->toplinks[(count($this->getToplinks())-1)]['id'] == $page['toplink']['id'] )
	    {
		return false;
	    }
	} else {
	    //Okay, sublink. Is it the last one?
	    if( $this->sublinks[(count($this->getSublinks())-1)]['id'] == $page['sublink']['id'] )
	    {
		return false;
	    }
	}
	//No impediments found...
	return true;
    }
    
    public function pageMoveUp($toplink, $sublink=null)
    {
	if( !$this->pageCanMoveUp($toplink, $sublink) )
	{
	    return false;
	}
	$page = $this->fetchLinks($toplink, $sublink);
	//Is this a toplink?
	if( empty($page['sublink']) )
	{
	    //Swap "order" with page above
	    $stmt = $this->db->prepare('SELECT `id`, `order` FROM `toplinks` WHERE `order` <= :order ORDER BY `order` DESC LIMIT 2');
	    $stmt->bindValue(':order', $page['toplink']['order'], PDO::PARAM_INT);
	    $stmt->execute();
	    $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
	    $stmt = $this->db->prepare('UPDATE `toplinks` SET `order` = :order WHERE `id` = :id');
	    $stmt->bindValue(':order',	$results[0]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[1]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    $stmt->bindValue(':order',	$results[1]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[0]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	} else {
	    //Swap "order" with page above
	    $stmt = $this->db->prepare('SELECT `id`, `order` FROM `sublinks` WHERE `order` <= :order AND `parent` = :parent ORDER BY `order` DESC LIMIT 2');
	    $stmt->bindValue(':order', $page['sublink']['order'], PDO::PARAM_INT);
	    $stmt->bindValue(':parent', $page['toplink']['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
	    $stmt = $this->db->prepare('UPDATE `sublinks` SET `order` = :order WHERE `id` = :id');
	    $stmt->bindValue(':order',	$results[0]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[1]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    
	    $stmt->bindValue(':order',	$results[1]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[0]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	}
	return true;
    }
    
    public function pageMoveDown($toplink, $sublink=null)
    {
	if( !$this->pageCanMoveDown($toplink, $sublink) )
	{
	    return false;
	}
	$page = $this->fetchLinks($toplink, $sublink);
	//Is this a toplink?
	if( empty($page['sublink']) )
	{
	    //Swap "order" with page above
	    $stmt = $this->db->prepare('SELECT `id`, `order` FROM `toplinks` WHERE `order` >= :order ORDER BY `order` ASC LIMIT 2');
	    $stmt->bindValue(':order', $page['toplink']['order'], PDO::PARAM_INT);
	    $stmt->execute();
	    $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
	    $stmt = $this->db->prepare('UPDATE `toplinks` SET `order` = :order WHERE `id` = :id');
	    $stmt->bindValue(':order',	$results[0]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[1]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    $stmt->bindValue(':order',	$results[1]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[0]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	} else {
	    //Swap "order" with page above
	    $stmt = $this->db->prepare('SELECT `id`, `order` FROM `sublinks` WHERE `order` >= :order AND `parent` = :parent ORDER BY `order` ASC LIMIT 2');
	    $stmt->bindValue(':order', $page['sublink']['order'], PDO::PARAM_INT);
	    $stmt->bindValue(':parent', $page['toplink']['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
	    $stmt = $this->db->prepare('UPDATE `sublinks` SET `order` = :order WHERE `id` = :id');
	    $stmt->bindValue(':order',	$results[0]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[1]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	    
	    $stmt->bindValue(':order',	$results[1]['order'],	PDO::PARAM_INT);
	    $stmt->bindValue(':id', 	$results[0]['id'],	PDO::PARAM_INT);
	    $stmt->execute();
	}
	return true;
    }
    
    public function pageCanDelete($toplink, $sublink=null)
    {
	//Get page details...
	$page = $this->fetchLinks($toplink, $sublink);
	if( !$page )
	{
	    return false; //Page doesn't exist..
	}
	//Is this a toplink? (Sublinks can always be deleted...)
	if( empty($page['sublink']) )
	{
	    //Is it "Home"? (Undeleteable!, Always ID 1)
	    if( $page['toplink']['id'] == 1 )
	    {
		return false;
	    }
	    //Does it have sublinks? They have to be deleted first..
	    $results = $this->db->prepare('SELECT * FROM `sublinks` WHERE `parent` = :parent ORDER BY `order`');
	    $results->bindValue(':parent', $page['toplink']['id']);
	    $results->execute();
	    $results = $results->fetchAll(PDO::FETCH_ASSOC);
	    if( !empty($results) )
	    {
		return false;
	    }
	}
	//No impediments found...
	return true;
    }
    
    public function createSublink($toplink)
    {
	if( !$this->pageExists($toplink) )
	{
	    return false;
	}
	//Get toplink ID
	$toplink = $this->fetchLinks($toplink);
	$toplink = $toplink['toplink'];
	//Prevent creating sublink for "HOME"
	if( $toplink['id'] == 1 )
	{
	    return false;
	}
	//Get current max ID
	$result = $this->db->query('SELECT MAX(`id`) FROM `sublinks`');
	$result = $result->fetch(PDO::FETCH_NUM);
	$new_id = $result[0];
	$new_id++;
	//Get last index of current order
	$result = $this->db->prepare('SELECT MAX(`order`) FROM `sublinks` WHERE `parent` = :parent');
	$result->bindValue(':parent', $toplink['id']);
	$result->execute();
	$result = $result->fetch(PDO::FETCH_NUM);
	$new_order = $result[0];
	$new_order++;
	//Make new sublink
	$result = $this->db->prepare('INSERT INTO `sublinks` VALUES (:id, :parent, :name, :identifier, :order)');
	$result->bindValue(':id',	$new_id,	PDO::PARAM_INT);
	$result->bindValue(':parent',	$toplink['id'],	PDO::PARAM_INT);
	$result->bindValue(':name',	'New Page',	PDO::PARAM_STR);
	$result->bindValue(':identifier', '-'.$new_id,	PDO::PARAM_STR);
	$result->bindValue(':order',	$new_order,	PDO::PARAM_INT);
	$result->execute();
	//Create empty page contents..
	$result = $this->db->query('SELECT MAX(`id`) FROM `pages`');
	$result = $result->fetch(PDO::FETCH_NUM);
	$page_id = $result[0];
	$page_id++;
	$result = $this->db->prepare('INSERT INTO `pages` (`id`, `sublink`, `content`, `is_include`) VALUES (:id, :sublink, :content, 0)');
	$result->bindValue(':id',	$page_id,	PDO::PARAM_INT);
	$result->bindValue(':sublink',	$new_id,	PDO::PARAM_INT);
	$result->bindValue(':content',	'',		PDO::PARAM_STR);
	$result->execute();
	return true;
    }
    
    public function updatePage($contents, $toplink, $sublink=null)
    {
	//Get current page details..
	$page = $this->fetchLinks($toplink, $sublink);
	if( !$page )
	{
	    return false; //Page doesn't exist
	}
	
	// CHECK IF IDENTIFIER EXISTS ALREADY (IF IT HAS CHANGED...)
	if( isset($contents['identifier']) && empty($page['sublink']) )
	{
	    if( $page['toplink']['identifier'] != $contents['identifier'] )
	    {
		if( $this->pageExists($contents['identifier']) )
		{
		    //Already in use, prevent it from being changed..
		    unset($contents['identifier']);
		}
	    }
	} else {
	    if( $page['sublink']['identifier'] != $contents['identifier'] )
	    {
		if( $this->pageExists($page['toplink']['identifier'], $contents['identifier']) )
		{
		    //Already in use, prevent it from being changed..
		    unset($contents['identifier']);
		}
	    }
	}
	
	
	//Build array for update from current contents
	$page_contents = array();
	if( empty($page['sublink']) )
	{
	    $page_contents['id'] = $page['toplink']['id'];
	    $page_contents['name'] = $page['toplink']['name'];
	    $page_contents['identifier'] = $page['toplink']['identifier'];
	    
	    $stmt = $this->db->prepare('SELECT * FROM `pages` WHERE `toplink` = :id');
	    $stmt->bindValue(':id', $page['toplink']['id']);
	    $stmt->execute();
	    $result = $stmt->fetch(PDO::FETCH_ASSOC);
	} else {
	    $page_contents['id'] = $page['sublink']['id'];
	    $page_contents['name'] = $page['sublink']['name'];
	    $page_contents['identifier'] = $page['sublink']['identifier'];
	    
	    $stmt = $this->db->prepare('SELECT * FROM `pages` WHERE `sublink` = :id');
	    $stmt->bindValue(':id', $page['sublink']['id']);
	    $stmt->execute();
	    $result = $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
	$page_contents['page_id'] = $result['id'];
	$page_contents['content'] = $result['content'];
	$page_contents['is_include'] = $result['is_include'];
	
	//Overwrite for variables in $contents other than "id" and "page_id"..
	foreach($contents as $key => $value)
	{
	    if( $key != 'id' && $key != 'page_id' )
	    {
		$page_contents[$key] = $value;
	    }
	}
	//Make sure is_include is 1 or 0
	$page_contents['is_include'] = (int) ( (bool) $page_contents['is_include'] );
	
	//Update database...
	$stmt = $this->db->prepare('UPDATE `pages` SET `content` = :content, `is_include` = :is_include WHERE `id` = :id');
	$stmt->bindValue(':content',	$page_contents['content'],	PDO::PARAM_STR);
	$stmt->bindValue(':is_include',	$page_contents['is_include'],	PDO::PARAM_INT);
	$stmt->bindValue(':id',		$page_contents['page_id'],	PDO::PARAM_INT);
	$stmt->execute();
	if( empty($page['sublink']) )
	{
	    $stmt = $this->db->prepare('UPDATE `toplinks` SET `name` = :name, `identifier` = :identifier WHERE `id` = :id');
	    $stmt->bindValue(':name',		$page_contents['name'],		PDO::PARAM_STR);
	    $stmt->bindValue(':identifier',	$page_contents['identifier'],	PDO::PARAM_STR);
	    $stmt->bindValue(':id',		$page_contents['id'],		PDO::PARAM_INT);
	    $stmt->execute();
	} else {	    
	    $stmt = $this->db->prepare('UPDATE `sublinks` SET `name` = :name, `identifier` = :identifier WHERE `id` = :id');
	    $stmt->bindValue(':name',		$page_contents['name'],		PDO::PARAM_STR);
	    $stmt->bindValue(':identifier',	$page_contents['identifier'],	PDO::PARAM_STR);
	    $stmt->bindValue(':id',		$page_contents['id'],		PDO::PARAM_INT);
	    $stmt->execute();
	}
	return true;
    }
    
    public function createToplink()
    {
	//Get current max ID
	$result = $this->db->query('SELECT MAX(`id`) FROM `toplinks`');
	$result = $result->fetch(PDO::FETCH_NUM);
	$new_id = $result[0];
	$new_id++;
	//Get last index of current order
	$result = $this->db->query('SELECT MAX(`order`) FROM `toplinks`');
	$result = $result->fetch(PDO::FETCH_NUM);
	$new_order = $result[0];
	$new_order++;
	//Make new toplink
	$result = $this->db->prepare('INSERT INTO `toplinks` VALUES (:id, :name, :identifier, :order)');
	$result->bindValue(':id',	$new_id,	PDO::PARAM_INT);
	$result->bindValue(':name',	'New Page',	PDO::PARAM_STR);
	$result->bindValue(':identifier', '-'.$new_id,	PDO::PARAM_STR);
	$result->bindValue(':order',	$new_order,	PDO::PARAM_INT);
	$result->execute();
	//Create empty page contents..
	$result = $this->db->query('SELECT MAX(`id`) FROM `pages`');
	$result = $result->fetch(PDO::FETCH_NUM);
	$page_id = $result[0];
	$page_id++;
	$result = $this->db->prepare('INSERT INTO `pages` (`id`, `toplink`, `content`, `is_include`) VALUES (:id, :toplink, :content, 0)');
	$result->bindValue(':id',	$page_id,	PDO::PARAM_INT);
	$result->bindValue(':toplink',	$new_id,	PDO::PARAM_INT);
	$result->bindValue(':content',	'',		PDO::PARAM_STR);
	$result->execute();
	return true;
    }
    
    public function deleteToplink($toplink)
    {
	if( !$this->pageCanDelete($toplink) )
	{
	    return false;
	}
	//Get ID
	$id = $this->fetchLinks($toplink);
	$id = $id['toplink']['id'];
	$result = $this->db->prepare('DELETE FROM `toplinks` WHERE `id` = :id');
	$result->bindValue(':id', $id, PDO::PARAM_INT);
	$result->execute();
	$result = $this->db->prepare('DELETE FROM `pages` WHERE `toplink` = :toplink');
	$result->bindValue(':toplink', $id, PDO::PARAM_INT);
	$result->execute();
	return true;
    }
    
    public function deleteSublink($toplink, $sublink)
    {
	if( !$this->pageCanDelete($toplink, $sublink) )
	{
	    return false;
	}
	//Get ID
	$id = $this->fetchLinks($toplink, $sublink);
	$id = $id['sublink']['id'];
	$result = $this->db->prepare('DELETE FROM `sublinks` WHERE `id` = :id');
	$result->bindValue(':id', $id, PDO::PARAM_INT);
	$result->execute();
	$result = $this->db->prepare('DELETE FROM `pages` WHERE `sublink` = :sublink');
	$result->bindValue(':sublink', $id, PDO::PARAM_INT);
	$result->execute();
	return true;
    }
}
?>